use strict;
use warnings;
use Test::More;

# generated by Dist::Zilla::Plugin::Test::PodSpelling 2.007005
use Test::Spelling 0.12;
use Pod::Wordlist;


add_stopwords(<DATA>);
all_pod_files_spelling_ok( qw( bin lib ) );
__DATA__
Alders
Alders'
App
AskPullRequestQuestions
ChangedFiles
ChangedFilesFactory
Cronin
Dave
Eilam
Eilam's
ExampleFileNameCheck
FileInspector
Florian
Fowler
GHE
GHPT
Greg
Inc
Kevin
MAXMIND
MAXMIND's
Mark
MaxMind
MaxMind's
Meta
OAuth
Oschwald
Oschwald's
OurMoose
OurMooseX
Ourperl
PRs
Parameterizable
Parameterized
Patrick
PayPal
Phair
Question
Ragwitz
Role
Rolsky
Rolsky's
Str
Strict
Trait
Types
WorkSubmitter
Wrapper
autarch
bin
gh
goschwald
iff
lib
mark
mfowler
pcronin
phair
rafl
